use color_eyre::Result;
use figment::providers::{Format, Toml};
use figment::Figment;
use serde::Deserialize;
use std::collections::HashMap;
use async_std::task;
use color_eyre::eyre::WrapErr;
use once_cell::sync::Lazy;
use regex::Regex;
use validator::Validate;

#[derive(Deserialize, Debug, Validate)]
pub struct Config {
    pub sites_directory: String,
    pub sockets_directory: String,
    #[validate(nested)]
    pub scopes: HashMap<String, ConfigScope>
}

static SECRET_PATTERN: Lazy<Regex> = Lazy::new(|| Regex::new(r"^[a-zA-Z0-9]*$").unwrap());

#[derive(Deserialize, Debug, Validate)]
pub struct ConfigScope {
    #[serde(with = "serde_regex")]
    pub domain_pattern: Regex,
    #[validate(length(equal = 100), regex(path = *SECRET_PATTERN))]
    pub secret: String
}

pub async fn load_config() -> Result<Config> {
    let config: Config = task::spawn_blocking(|| Figment::new()
        .merge(Toml::file("./config.toml"))
        .extract()
        .wrap_err("Failed to load the configuration.")).await?;

    config.validate().wrap_err("Failed to validate the configuration.")?;
    Ok(config)
}